# ---------------------------------------------------------------------------------------------------------------------
# Este script 1) Crea mosaicos con las escenas de ALOS PALSAR
#             2) Subtrae un área de interes en el mosaico creado
#             3) Convierte numero dijitales de las imagenes a valores de retrodispersion
#             4) se aplica el filtro de LEE para eliminar el ruido de la imagenes
#
# 
# Instalar los paquetes:
# rgdal, spatial.tools, sp
# raster, whitebox
#
# Instalar la libreria wbt requiere todas las lineas siguientes, 
# se realiza una vez en su computadora,
# si  ya se tiene instalado la libreria correctamente 
# no se requiere correr las 7 lineas siguientes

install.packages("devtools")
library(devtools)
devtools::install_github("giswqs/whiteboxR", force = TRUE)
whitebox::install_whitebox(force = TRUE)


#------------------------------------------------------------------------------------------------------
# PASO 1: PREPACION DE DATOS Y DESCOMPRESION DE ARCHIVOS 
#------------------------------------------------------------------------------------------------------
# Definir directorio de trabajo, este debe contener los archivos descargados del
# Global PALSAR-2/PALSAR/JERS-1 Mosaic and Forest/Non-Forest map (imagenes de ALOS PALSAR)
# recordar que el directorio de trabajo en este manual es "c:/Mapeo_biomasa/FCP"


setwd(choose.dir(default = "",
                 caption = "Selecciona el directorio de trabajo")) #FCP
getwd()
folder <- getwd()

# Abrir los siguientes paquetes
library(sp)
library(raster)
library(whitebox)
wbt_version()


# Cargar el pologono con el area de estudio (archivo shapefile)
area <- shapefile(choose.files(caption = "selecciona el shapefile 
                               con el pologono de estudio"))
plot(area)
summary(area)

# Crear lista de archivos comprimidos con extension tar.gz
list.gz<-list.files(path=".", pattern ="tar.gz")
list.gz

# Crear un folder para los archivos a descomprimir
Name.Dir.Decompress <- "Files_AlosPalsar" # Nombre del directorio

{ifelse(!
          dir.exists(file.path(folder,Name.Dir.Decompress)),
        dir.create(file.path(folder,Name.Dir.Decompress)),
        FALSE)
  dir.files <- file.path(folder,Name.Dir.Decompress)}

# Descomprimir todos los archivos con la funcion untar()
for (i in 1:length(list.gz)){
  untar(list.gz[i], exdir = dir.files)
}
# Crear lista de archivos descomprimidos unicamente los que contengan las polarizaciones HH y HV
# ambos pueden definirse por los ultimos caracteres del nombre del archivo
list.rast <- list.files(path = dir.files, 
                               pattern = "HH_F02DAR$|HV_F02DAR$",
                        full.names = TRUE)
list.rast

# PASO 2: CREAR EL MOSAICO Y RECORTAR EL AREA DE ESTUDIO
#------------------------------------------------------------------------------------------------------
# Primero se debe homogenizar proyeccion del area de estudio con la de los rasters de ALOS PALSAR
# Visualizar el poligono 
areaNew <- spTransform(area, CRS(projection(raster(list.rast[1]))))
plot(areaNew)
summary(areaNew)

# Crear un mosaico con las escenas
# Listar los nombres de los archivos por tipo de imagen
FileHH <- list.files(path = dir.files, pattern = "HH_F02DAR$",
                     full.names = TRUE) #HH polarizacion 
FileHH
FileHV <- list.files(path = dir.files, pattern = "HV_F02DAR$",
                     full.names = TRUE) #HV polarizacion
FileHV
FileFNF <- list.files(path = dir.files, pattern = "C_F02DAR$",
                      full.names = TRUE) #Mascara FNF
FileFNF

# Definir la lista de nombres de archivos en formato raster
HH.list <- list() #se crea una lista vacia
for(i in 1:length(FileHH)){  
  HH.list[[i]] <- raster(FileHH[i])  #files a raster
} 
HV.list <- list()
for(i in 1:length(FileHV)){  
  HV.list[[i]] <- raster(FileHV[i])  
}
FNF.list <- list()
for(i in 1:length(FileFNF)){  
  FNF.list[[i]] <- raster(FileFNF[i])  
}



# Unir los raster a un nuevo raster layer
HH.merge <- do.call(merge, HH.list)
HV.merge <- do.call(merge, HV.list)
FNF.merge <- do.call(merge, FNF.list)

plot(HV.merge, col = gray.colors(30, start = 0.3, end = 1),  legend = FALSE)
plot(areaNew, add = TRUE, lwd = 2, border = "yellow")

# Extraer el area de estudio en los mosaicos creados anteriomente
HH_crop <- crop(HH.merge, areaNew)
HV_crop <- crop(HV.merge, areaNew)
FNF_crop <- crop(FNF.merge, areaNew)

plot(HH_crop, col = gray.colors(30, start = 0.3, end = 1), legend = FALSE)
plot(HV_crop, col = gray.colors(30, start = 0.3, end = 1), legend = FALSE)
plot(FNF_crop, col= c("darkgreen","khaki1","navyblue"), legend = FALSE)


# PASO 3: CONVERTIR VALORES DE NUMEOS DIGITALES A RETRODISPERSION
# ------------------------------------------------------------------------------------------------------
# Definir un directorio de salida de archivos
{Name.Dir.Bcks <- "Bcks" # Nombre del directorio
{ifelse(!
          dir.exists(file.path(folder,Name.Dir.Bcks)),
        dir.create(file.path(folder,Name.Dir.Bcks)),
        FALSE)
  dir.Bcks <- file.path(folder,Name.Dir.Bcks)}
}

# Definir la funcion de calculo 
fun<- function (x) { 
  (10 * log10(x ^ 2) - 83)
}

# Calc() es una funcion de calculo para raster a nuevos raster mediante una funcion
HH.bck <- calc(HH_crop,fun,
               filename =  paste(dir.Bcks,"/","HH", sep = ""),
               overwrite =TRUE,
               format="GTiff")# Guardar archivos en formato Gtiff
HV.bck <- calc(HV_crop,fun,
               filename =  paste(dir.Bcks,"/","HV", sep = ""),
               overwrite =TRUE,
               format="GTiff")

plot(HH.bck,
     col = gray.colors(30, start = 0.3, end = 1),
     legend = TRUE)
plot(HV.bck,
     col = gray.colors(30, start = 0.3, end = 1),
     legend = TRUE)

#PASO 4: APLICAR EL FILTRO DE LEE PARA REDUCIR EL RUIDO DE LAS IMAGENES
# ------------------------------------------------------------------------------------------------------
#crear directorio para guardar los archivos
Name.Dir.Lee <- "LeeFl" # Nombre del directorio

{ifelse(!
          dir.exists(file.path(folder,Name.Dir.Lee)),
        dir.create(file.path(folder, Name.Dir.Lee)),
        FALSE)
  dir.LeeFl <- file.path(folder,Name.Dir.Lee)
}


BckList <- list.files(path = dir.Bcks, pattern = ".tif$")
BckList

for (i in 1:length(BckList)){
  wbt_lee_sigma_filter(file.path(dir.Bcks,BckList[i]), 
                       file.path( dir.LeeFl,BckList[i]),
                       filterx =3,
                       filtery =3)
}

# PASO 5: CONVERTIR LAS COORDENADAS DE LAS IMAGENES PRE-PROCESADAS  A PROYECTADAS
# ------------------------------------------------------------------------------------------------------
#crear el directorio de salida
Name.Dir.IMG <- "IMG" # Nombre del directorio
{ifelse(!
          dir.exists(file.path(folder,Name.Dir.IMG)),
        dir.create(file.path(folder,Name.Dir.IMG)), FALSE
)
  dir.IMG <- file.path(folder,Name.Dir.IMG)
}

FileList <- list.files(path=dir.LeeFl, pattern = ".tif$",
                       full.names = TRUE)
FileList


# Definir la nueva proyeccion en formato PROJ4
# Mexico ITRF2008/LCC EPSG: 6372
newProj <- "+proj=lcc +lat_1=17.5 +lat_2=29.5 +lat_0=12 +lon_0=-102 +x_0=2500000 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
#crs(newProj)

# Cambiar sistema de coordenadas de los archivos en lista FileList ( HH y HV)
proj <- list()

for (i in 1:length(FileList)){
  proj[[i]] <- projectRaster(raster(FileList[i]), crs= newProj)
}
proj


# Guardar la capa HH y HV en un raster multibanda

stack <- stack(proj)
stack

terra::writeRaster(stack,
            filename = file.path(dir.IMG,"FCP.tif"),
            overwrite = TRUE,
            format = "GTiff")

# Visualizacion imagen SAR como un compuesto RGB
plotRGB(stack, r=1, g=2, b=1,axes=TRUE, stretch ="lin" )

# Proyeccion de la imagen FNF recortada
projectRaster(FNF_crop,
              crs = newProj,
              filename = paste(dir.IMG,"FNF.tif", sep= "/"),
              overwrite = TRUE,
              format = "GTiff") # Guardar en formato tiff

